@@ -3,8 +3,10 @@ from conductor.client.http.api_client import ApiClient
 from conductor.client.http.api.task_resource_api import TaskResourceApi
 from conductor.client.http.models.task import Task
 from conductor.client.http.models.task_result import TaskResult
+from conductor.client.telemetry.metrics_collector import MetricsCollector
 from conductor.client.worker.worker_interface import WorkerInterface
 import logging
+import sys
 import time
 
 logger = logging.getLogger(
@@ -15,6 +17,8 @@ logger = logging.getLogger(
 
 
 class TaskRunner:
+    metrics_collector = MetricsCollector()
+
     def __init__(self, worker: WorkerInterface, configuration: Configuration = None):
         if configuration != None and not isinstance(configuration, Configuration):
             raise Exception('Invalid configuration')
@@ -38,55 +42,81 @@ class TaskRunner:
 
     def __poll_task(self) -> Task:
         task_definition_name = self.worker.get_task_definition_name()
+        self.metrics_collector.increment_task_poll(
+            task_definition_name
+        )
         logger.info(f'Polling task for: {task_definition_name}')
         try:
+            start_time = time.time()
             task = self.__get_task_resource_api().poll(
                 tasktype=task_definition_name
             )
-        except Exception:
-            return None
-        message = 'Polled task for worker: {task_definition_name}, identity: {identity}'
-        logger.debug(
-            message.format(
-                task_definition_name=task_definition_name,
-                identity=self.worker.get_identity()
+            finish_time = time.time()
+            time_spent = finish_time - start_time
+            self.metrics_collector.record_task_poll_time(
+                task_definition_name, time_spent
             )
+        except Exception as e:
+            self.metrics_collector.increment_task_poll_error(
+                task_definition_name, type(e)
+            )
+            logger.warning(
+                f'Failed to poll task for: {task_definition_name}, reason: {e}'
+            )
+            return None
+        logger.debug(
+            f'Polled task: {task_definition_name}, worker_id: {self.worker.get_identity()}'
         )
         return task
 
     def __execute_task(self, task: Task) -> TaskResult:
         if not isinstance(task, Task):
             return None
+        task_definition_name = self.worker.get_task_definition_name()
         logger.info(
-            'Executing task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}'.format(
+            'Executing task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}'.format(
                 task_id=task.task_id,
                 workflow_instance_id=task.workflow_instance_id,
-                worker_name=self.worker.get_task_definition_name()
+                task_definition_name=task_definition_name
             )
         )
         try:
+            start_time = time.time()
             task_result = self.worker.execute(task)
+            finish_time = time.time()
+            time_spent = finish_time - start_time
+            self.metrics_collector.record_task_execute_time(
+                task_definition_name,
+                time_spent
+            )
+            self.metrics_collector.record_task_result_payload_size(
+                task_definition_name,
+                sys.getsizeof(task_result)
+            )
             logger.info(
-                'Executed task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}'.format(
+                'Executed task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}'.format(
                     task_id=task.task_id,
                     workflow_instance_id=task.workflow_instance_id,
-                    worker_name=self.worker.get_task_definition_name()
+                    task_definition_name=task_definition_name
                 )
             )
         except Exception as e:
+            self.metrics_collector.increment_task_execution_error(
+                task_definition_name, type(e)
+            )
             task_result = TaskResult(
                 task_id=task.task_id,
                 workflow_instance_id=task.workflow_instance_id,
-                worker_id=self.worker.get_task_definition_name()
+                worker_id=self.worker.get_identity()
             )
             task_result.status = 'FAILED'
             task_result.reason_for_incompletion = str(e)
             logger.warning(
-                'Failed to execute task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}, reason: {reason}'.format(
+                'Failed to execute task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}, reason: {reason}'.format(
                     task_id=task.task_id,
                     workflow_instance_id=task.workflow_instance_id,
-                    worker_name=self.worker.get_task_definition_name(),
-                    reason=str(e)
+                    task_definition_name=task_definition_name,
+                    reason=e
                 )
             )
         return task_result
@@ -94,11 +124,12 @@ class TaskRunner:
     def __update_task(self, task_result: TaskResult):
         if not isinstance(task_result, TaskResult):
             return None
+        task_definition_name = self.worker.get_task_definition_name()
         logger.debug(
-            'Updating task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}'.format(
+            'Updating task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}'.format(
                 task_id=task_result.task_id,
                 workflow_instance_id=task_result.workflow_instance_id,
-                worker_name=self.worker.get_task_definition_name()
+                task_definition_name=task_definition_name
             )
         )
         try:
@@ -106,20 +137,23 @@ class TaskRunner:
                 body=task_result
             )
         except Exception as e:
+            self.metrics_collector.increment_task_update_error(
+                task_definition_name, type(e)
+            )
             logger.warning(
-                'Failed to update task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}, reason: {reason}'.format(
+                'Failed to update task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}, reason: {reason}'.format(
                     task_id=task_result.task_id,
                     workflow_instance_id=task_result.workflow_instance_id,
-                    worker_name=self.worker.get_task_definition_name(),
-                    reason=str(e)
+                    task_definition_name=task_definition_name,
+                    reason=e
                 )
             )
             return None
         logger.info(
-            'Updated task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, worker: {worker_name}, response: {response}'.format(
+            'Updated task, id: {task_id}, workflow_instance_id: {workflow_instance_id}, task_definition_name: {task_definition_name}, response: {response}'.format(
                 task_id=task_result.task_id,
                 workflow_instance_id=task_result.workflow_instance_id,
-                worker_name=self.worker.get_task_definition_name(),
+                task_definition_name=task_definition_name,
                 response=str(response)
             )
         )
